@@ -12,10 +12,10 @@ static const char *blur_blend_vertex_shader =
     R"(
 #version 100
 
-attribute mediump vec2 position;
-attribute mediump vec2 uv_in;
+attribute highp vec2 position;
+attribute highp vec2 uv_in;
 
-varying mediump vec2 uvpos[2];
+varying highp vec2 uvpos[2];
 
 uniform mat4 mvp;
 uniform mat4 background_uv_matrix;
@@ -31,13 +31,13 @@ static const char *blur_blend_fragment_shader =
     R"(
 #version 100
 @builtin_ext@
-precision mediump float;
+precision highp float;
 
 @builtin@
 uniform float sat;
 uniform sampler2D bg_texture;
 
-varying mediump vec2 uvpos[2];
+varying highp vec2 uvpos[2];
 
 vec3 saturation(vec3 rgb, float adjustment)
 {
@@ -74,20 +74,20 @@ wf_blur_base::wf_blur_base(std::string name)
     this->degrade_opt.set_callback(options_changed);
     this->iterations_opt.set_callback(options_changed);
 
-    OpenGL::render_begin();
-    blend_program.compile(blur_blend_vertex_shader, blur_blend_fragment_shader);
-    OpenGL::render_end();
+    wf::gles::run_in_context_if_gles([&]
+    {
+        blend_program.compile(blur_blend_vertex_shader, blur_blend_fragment_shader);
+    });
 }
 
 wf_blur_base::~wf_blur_base()
 {
-    OpenGL::render_begin();
-    fb[0].release();
-    fb[1].release();
-    program[0].free_resources();
-    program[1].free_resources();
-    blend_program.free_resources();
-    OpenGL::render_end();
+    wf::gles::run_in_context_if_gles([&]
+    {
+        program[0].free_resources();
+        program[1].free_resources();
+        blend_program.free_resources();
+    });
 }
 
 int wf_blur_base::calculate_blur_radius()
@@ -96,7 +96,7 @@ int wf_blur_base::calculate_blur_radius()
 }
 
 void wf_blur_base::render_iteration(wf::region_t blur_region,
-    wf::framebuffer_t& in, wf::framebuffer_t& out,
+    wf::auxilliary_buffer_t& in, wf::auxilliary_buffer_t& out,
     int width, int height)
 {
     /* Special case for small regions where we can't really blur, because we
@@ -104,13 +104,16 @@ void wf_blur_base::render_iteration(wf::region_t blur_region,
     width  = std::max(width, 1);
     height = std::max(height, 1);
 
-    out.allocate(width, height);
-    out.bind();
+    out.allocate({width, height});
 
-    GL_CALL(glBindTexture(GL_TEXTURE_2D, in.tex));
+    GLuint tex_id = wf::gles_texture_t::from_aux(in).tex_id;
+
+    wf::gles::bind_render_buffer(out.get_renderbuffer());
+    GL_CALL(glActiveTexture(GL_TEXTURE0));
+    GL_CALL(glBindTexture(GL_TEXTURE_2D, tex_id));
     for (auto& b : blur_region)
     {
-        out.scissor(wlr_box_from_pixman_box(b));
+        wf::gles::scissor_render_buffer(out.get_renderbuffer(), wlr_box_from_pixman_box(b));
         GL_CALL(glDrawArrays(GL_TRIANGLE_FAN, 0, 4));
     }
 }
@@ -147,7 +150,7 @@ static wf::geometry_t sanitize(wf::geometry_t box, int degrade,
     return wf::clamp(out_box, bounds);
 }
 
-wlr_box wf_blur_base::copy_region(wf::framebuffer_t& result,
+wlr_box wf_blur_base::copy_region(wf::auxilliary_buffer_t& result,
     const wf::render_target_t& source, const wf::region_t& region)
 {
     auto subbox = source.framebuffer_box_from_geometry_box(
@@ -161,18 +164,18 @@ wlr_box wf_blur_base::copy_region(wf::framebuffer_t& result,
     subbox = sanitize(subbox, degrade_opt, source_box);
     int degraded_width  = subbox.width / degrade_opt;
     int degraded_height = subbox.height / degrade_opt;
+    result.allocate({degraded_width, degraded_height});
 
-    OpenGL::render_begin(source);
-    result.allocate(degraded_width, degraded_height);
+    GLuint src_fb = wf::gles::ensure_render_buffer_fb_id(source);
+    GLuint dst_fb = wf::gles::ensure_render_buffer_fb_id(result.get_renderbuffer());
+    GL_CALL(glBindFramebuffer(GL_READ_FRAMEBUFFER, src_fb));
+    GL_CALL(glBindFramebuffer(GL_DRAW_FRAMEBUFFER, dst_fb));
 
-    GL_CALL(glBindFramebuffer(GL_READ_FRAMEBUFFER, source.fb));
-    GL_CALL(glBindFramebuffer(GL_DRAW_FRAMEBUFFER, result.fb));
     GL_CALL(glBlitFramebuffer(
-        subbox.x, source.viewport_height - subbox.y - subbox.height,
-        subbox.x + subbox.width, source.viewport_height - subbox.y,
+        subbox.x, subbox.y,
+        subbox.x + subbox.width, subbox.y + subbox.height,
         0, 0, degraded_width, degraded_height,
-        GL_COLOR_BUFFER_BIT, GL_LINEAR));
-    OpenGL::render_end();
+        GL_COLOR_BUFFER_BIT, GL_NEAREST));
 
     return subbox;
 }
@@ -201,8 +204,7 @@ void wf_blur_base::prepare_blur(const wf::render_target_t& target_fb, const wf::
     blur_damage += -wf::point_t{damage_box.x, damage_box.y};
     blur_damage *= 1.0 / degrade;
 
-    int r = blur_fb0(blur_damage, fb[0].viewport_width, fb[0].viewport_height);
-
+    int r = blur_fb0(blur_damage, fb[0].get_size().width, fb[0].get_size().height);
     /* Make sure the result is always fb[0], because that's what is used in render()
      * */
     if (r != 0)
@@ -218,10 +220,11 @@ static wf::pointf_t get_center(wf::geometry_t g)
     return {g.x + g.width / 2.0, g.y + g.height / 2.0};
 }
 
-void wf_blur_base::render(wf::texture_t src_tex, wlr_box src_box, const wf::region_t& damage,
+void wf_blur_base::render(wf::gles_texture_t src_tex, wlr_box src_box, const wf::region_t& damage,
     const wf::render_target_t& background_source_fb, const wf::render_target_t& target_fb)
 {
-    OpenGL::render_begin(target_fb);
+    wf::gles_texture_t blurred_background = wf::gles_texture_t::from_aux(fb[0]);
+    wf::gles::ensure_render_buffer_fb_id(target_fb);
     blend_program.use(src_tex.type);
 
     /* Use shader and enable vertex and texcoord data */
@@ -256,7 +259,7 @@ void wf_blur_base::render(wf::texture_t src_tex, wlr_box src_box, const wf::regi
     auto blurred_box = prepared_geometry;
     // prepared_geometry is the projected damage bounding box
 
-    glm::mat4 fb_fix   = target_fb.transform;
+    glm::mat4 fb_fix   = wf::gles::output_transform(target_fb);
     const auto scale_x = 1.0 * view_box.width / blurred_box.width;
     const auto scale_y = 1.0 * view_box.height / blurred_box.height;
     glm::mat4 scale    = glm::scale(glm::mat4(1.0), glm::vec3{scale_x, scale_y, 1.0});
@@ -264,26 +267,27 @@ void wf_blur_base::render(wf::texture_t src_tex, wlr_box src_box, const wf::regi
     const wf::pointf_t center_view     = get_center(view_box);
     const wf::pointf_t center_prepared = get_center(blurred_box);
     const auto translate_x = 1.0 * (center_view.x - center_prepared.x) / view_box.width;
-    const auto translate_y = -1.0 * (center_view.y - center_prepared.y) / view_box.height;
+    const auto translate_y = 1.0 * (center_view.y - center_prepared.y) / view_box.height;
     glm::mat4 fix_center   = glm::translate(glm::mat4(1.0), glm::vec3{translate_x, translate_y, 0.0});
     glm::mat4 composite    = scale * fix_center * fb_fix;
     blend_program.uniformMatrix4f("background_uv_matrix", composite);
 
     /* Blend blurred background with window texture src_tex */
-    blend_program.uniformMatrix4f("mvp", target_fb.get_orthographic_projection());
+    blend_program.uniformMatrix4f("mvp", wf::gles::render_target_orthographic_projection(target_fb));
     /* XXX: core should give us the number of texture units used */
     blend_program.uniform1i("bg_texture", 1);
     blend_program.uniform1f("sat", saturation_opt);
 
     blend_program.set_active_texture(src_tex);
     GL_CALL(glActiveTexture(GL_TEXTURE0 + 1));
-    GL_CALL(glBindTexture(GL_TEXTURE_2D, fb[0].tex));
+    GL_CALL(glBindTexture(GL_TEXTURE_2D, blurred_background.tex_id));
+
     /* Render it to target_fb */
-    target_fb.bind();
+    wf::gles::bind_render_buffer(target_fb);
 
     for (const auto& box : damage)
     {
-        target_fb.logic_scissor(wlr_box_from_pixman_box(box));
+        wf::gles::render_target_logic_scissor(target_fb, wlr_box_from_pixman_box(box));
         GL_CALL(glDrawArrays(GL_TRIANGLE_FAN, 0, 4));
     }
 
@@ -295,7 +299,6 @@ void wf_blur_base::render(wf::texture_t src_tex, wlr_box src_box, const wf::regi
     GL_CALL(glActiveTexture(GL_TEXTURE0));
     GL_CALL(glBindTexture(GL_TEXTURE_2D, 0));
     blend_program.deactivate();
-    OpenGL::render_end();
 }
 
 std::unique_ptr<wf_blur_base> create_blur_from_name(std::string algorithm_name)
