@@ -1,5 +1,6 @@
 #include <wayfire/per-output-plugin.hpp>
 #include <memory>
+#include <list>
 #include <wayfire/config/types.hpp>
 #include <wayfire/plugin.hpp>
 #include <wayfire/view.hpp>
@@ -20,6 +21,7 @@
 #include "wayfire/scene-render.hpp"
 #include "wayfire/scene.hpp"
 #include "wayfire/signal-provider.hpp"
+#include <list>
 
 using blur_algorithm_provider =
     std::function<nonstd::observer_ptr<wf_blur_base>()>;
@@ -50,17 +52,6 @@ class blur_node_t : public transformer_base_node_t
         this->provider = provider;
     }
 
-    ~blur_node_t()
-    {
-        OpenGL::render_begin();
-        for (auto& buffer : saved_pixels)
-        {
-            buffer.pixels.release();
-        }
-
-        OpenGL::render_end();
-    }
-
     std::string stringify() const override
     {
         return "blur";
@@ -71,7 +62,7 @@ class blur_node_t : public transformer_base_node_t
 
     struct saved_pixels_t
     {
-        wf::framebuffer_t pixels;
+        wf::auxilliary_buffer_t pixels;
         wf::region_t region;
         bool taken = false;
     };
@@ -183,22 +174,27 @@ class blur_render_instance_t : public transformer_render_instance_t<blur_node_t>
         // Nodes below should re-render the padded areas so that we can sample from them
         damage |= padded_region;
 
-        OpenGL::render_begin();
-        saved_pixels->pixels.allocate(target.viewport_width, target.viewport_height);
-        saved_pixels->pixels.bind();
-        GL_CALL(glBindFramebuffer(GL_READ_FRAMEBUFFER, target.fb));
+        saved_pixels->pixels.allocate(target.get_size());
 
-        /* Copy pixels in padded_region from target_fb to saved_pixels. */
-        for (const auto& box : saved_pixels->region)
+        wf::gles::run_in_context_if_gles([&]
         {
-            GL_CALL(glBlitFramebuffer(
-                box.x1, target.viewport_height - box.y2,
-                box.x2, target.viewport_height - box.y1,
-                box.x1, box.y1, box.x2, box.y2,
-                GL_COLOR_BUFFER_BIT, GL_LINEAR));
-        }
+            GLuint target_fb = wf::gles::ensure_render_buffer_fb_id(target);
+
+            wf::gles::bind_render_buffer(saved_pixels->pixels.get_renderbuffer());
+            GL_CALL(glBindFramebuffer(GL_READ_FRAMEBUFFER, target_fb));
+
+            /* Copy pixels in padded_region from target_fb to saved_pixels. */
+            for (const auto& box : saved_pixels->region)
+            {
+                GL_CALL(glBlitFramebuffer(
+                    box.x1, box.y1,
+                    box.x2, box.y2,
+                    box.x1, box.y1,
+                    box.x2, box.y2,
+                    GL_COLOR_BUFFER_BIT, GL_LINEAR));
+            }
+        });
 
-        OpenGL::render_end();
         instructions.push_back(render_instruction_t{
                     .instance = this,
                     .target   = target,
@@ -206,39 +202,45 @@ class blur_render_instance_t : public transformer_render_instance_t<blur_node_t>
                 });
     }
 
-    void render(const wf::render_target_t& target, const wf::region_t& damage) override
+    void render(const wf::scene::render_instruction_t& data) override
     {
-        auto tex = get_texture(target.scale);
         auto bounding_box = self->get_bounding_box();
-        if (!damage.empty())
+        data.pass->custom_gles_subpass([&]
         {
-            auto translucent_damage = calculate_translucent_damage(target, damage);
-            self->provider()->prepare_blur(target, translucent_damage);
-            self->provider()->render(tex, bounding_box, damage, target, target);
-        }
+            auto tex = wf::gles_texture_t{get_texture(data.target.scale)};
+            if (!data.damage.empty())
+            {
+                auto translucent_damage = calculate_translucent_damage(data.target, data.damage);
+                self->provider()->prepare_blur(data.target, translucent_damage);
+                self->provider()->render(tex, bounding_box, data.damage, data.target, data.target);
+            }
 
-        OpenGL::render_begin(target);
-        // Setup framebuffer I/O. target_fb contains the frame
-        // rendered with expanded damage and artifacts on the edges.
-        // saved_pixels has the the padded region of pixels to overwrite the
-        // artifacts that blurring has left behind.
-        GL_CALL(glBindFramebuffer(GL_READ_FRAMEBUFFER, saved_pixels->pixels.fb));
+            GL_CALL(glDisable(GL_SCISSOR_TEST));
 
-        /* Copy pixels back from saved_pixels to target_fb. */
-        for (const auto& box : saved_pixels->region)
-        {
-            GL_CALL(glBlitFramebuffer(
-                box.x1, box.y1, box.x2, box.y2,
-                box.x1, target.viewport_height - box.y2,
-                box.x2, target.viewport_height - box.y1,
-                GL_COLOR_BUFFER_BIT, GL_LINEAR));
-        }
+            GLuint saved_fb = wf::gles::ensure_render_buffer_fb_id(saved_pixels->pixels.get_renderbuffer());
+            wf::gles::bind_render_buffer(data.target);
+            // Setup framebuffer I/O. target_fb contains the frame
+            // rendered with expanded damage and artifacts on the edges.
+            // saved_pixels has the the padded region of pixels to overwrite the
+            // artifacts that blurring has left behind.
+            GL_CALL(glBindFramebuffer(GL_READ_FRAMEBUFFER, saved_fb));
 
-        /* Reset stuff */
-        saved_pixels->region.clear();
-        self->release_saved_pixel_buffer(saved_pixels);
-        saved_pixels = NULL;
-        OpenGL::render_end();
+            /* Copy pixels back from saved_pixels to target_fb. */
+            for (const auto& box : saved_pixels->region)
+            {
+                GL_CALL(glBlitFramebuffer(
+                    box.x1, box.y1,
+                    box.x2, box.y2,
+                    box.x1, box.y1,
+                    box.x2, box.y2,
+                    GL_COLOR_BUFFER_BIT, GL_LINEAR));
+            }
+
+            /* Reset stuff */
+            saved_pixels->region.clear();
+            self->release_saved_pixel_buffer(saved_pixels);
+            saved_pixels = NULL;
+        });
     }
 
     direct_scanout try_scanout(wf::output_t *output) override
@@ -267,17 +269,18 @@ class wayfire_blur : public wf::plugin_interface_t
     // This is needed, because when blurring, the pixels that changed
     // affect a larger area than the really damaged region, e.g. the region
     // that comes from client damage.
-    wf::signal::connection_t<wf::scene::render_pass_begin_signal>
-    on_render_pass_begin = [=] (wf::scene::render_pass_begin_signal *ev)
+    wf::signal::connection_t<wf::render_pass_begin_signal>
+    on_render_pass_begin = [=] (wf::render_pass_begin_signal *ev)
     {
         if (!provider)
         {
             return;
         }
 
-        const int padding = calculate_damage_padding(ev->target, provider()->calculate_blur_radius());
+        const int padding =
+            calculate_damage_padding(ev->pass.get_target(), provider()->calculate_blur_radius());
         ev->damage.expand_edges(padding);
-        ev->damage &= ev->target.geometry;
+        ev->damage &= ev->pass.get_target().geometry;
     };
 
   public:
@@ -331,6 +334,14 @@ class wayfire_blur : public wf::plugin_interface_t
   public:
     void init() override
     {
+        if (!wf::get_core().is_gles2())
+        {
+            const char *render_type =
+                wf::get_core().is_vulkan() ? "vulkan" : (wf::get_core().is_pixman() ? "pixman" : "unknown");
+            LOGE("blur: requires GLES2 support, but current renderer is ", render_type);
+            return;
+        }
+
         wf::get_core().connect(&on_render_pass_begin);
         blur_method_changed = [=] ()
         {
