@@ -9,7 +9,7 @@ class wf_cube_background_skydome : public wf_cube_background_base
 {
   public:
     wf_cube_background_skydome(wf::output_t *output);
-    virtual void render_frame(const wf::framebuffer_t& fb,
+    virtual void render_frame(const wf::render_target_t& fb,
         wf_cube_animation_attribs& attribs) override;
 
     virtual ~wf_cube_background_skydome();
