@@ -1,11 +1,11 @@
 #pragma once
 
-#include <string>
+#include <wayfire/scene-render.hpp>
 #include <wayfire/util.hpp>
 #include <wayfire/opengl.hpp>
 #include <wayfire/render-manager.hpp>
 #include <wayfire/util/duration.hpp>
-#include <wayfire/plugins/common/simple-texture.hpp>
+#include <wayfire/plugins/common/cairo-util.hpp>
 
 #include <cairo.h>
 #include <pango/pango.h>
@@ -36,7 +36,7 @@ class button_t
     button_t(const decoration_theme_t& theme,
         std::function<void()> damage_callback);
 
-    ~button_t() = default;
+    ~button_t();
     button_t(const button_t &) = delete;
     button_t(button_t &&) = delete;
     button_t& operator =(const button_t&) = delete;
@@ -67,19 +67,17 @@ class button_t
      * Render the button on the given framebuffer at the given coordinates.
      * Precondition: set_button_type() has been called, otherwise result is no-op
      *
-     * @param buffer The target framebuffer
+     * @param data The render data
      * @param geometry The geometry of the button, in logical coordinates
-     * @param scissor The scissor rectangle to render.
      */
-    void render(const wf::render_target_t& buffer, wf::geometry_t geometry,
-        wf::geometry_t scissor);
+    void render(const scene::render_instruction_t& data, wf::geometry_t geometry);
 
   private:
     const decoration_theme_t& theme;
 
     /* Whether the button needs repaint */
     button_type_t type;
-    wf::simple_texture_t button_texture;
+    wf::owned_texture_t button_texture;
 
     /* Whether the button is currently being hovered */
     bool is_hovered = false;
