@@ -1,12 +1,20 @@
 #pragma once
 
+#include "wayfire/signal-definitions.hpp"
+#include "wayfire/signal-provider.hpp"
 #include <string>
 
 #include <wayfire/plugin.hpp>
 #include <wayfire/output.hpp>
-
 #include <wayfire/plugins/scale-signal.hpp>
-#include <wayfire/plugins/scale-transform.hpp>
+
+namespace wf
+{
+namespace scene
+{
+class title_overlay_node_t;
+}
+}
 
 
 class scale_show_title_t
@@ -30,10 +38,13 @@ class scale_show_title_t
 
   protected:
     /* signals */
-    wf::signal_connection_t view_filter;
-    wf::signal_connection_t scale_end;
-    wf::signal_connection_t add_title_overlay;
-    wf::signal_connection_t mouse_update;
+    wf::signal::connection_t<scale_filter_signal> view_filter;
+    wf::signal::connection_t<scale_end_signal> scale_end;
+    wf::signal::connection_t<scale_transformer_added_signal> add_title_overlay;
+    wf::signal::connection_t<scale_transformer_removed_signal> rem_title_overlay;
+    wf::signal::connection_t<wf::post_input_event_signal<wlr_pointer_motion_event>> post_motion;
+    wf::signal::connection_t<wf::post_input_event_signal<wlr_pointer_motion_absolute_event>>
+    post_absolute_motion;
 
     enum class title_overlay_t
     {
@@ -42,7 +53,7 @@ class scale_show_title_t
         ALL,
     };
 
-    friend class view_title_overlay_t;
+    friend class wf::scene::title_overlay_node_t;
 
     title_overlay_t show_view_title_overlay;
     /* only used if title overlay is set to follow the mouse */
