@@ -138,14 +138,14 @@ class wayfire_move : public wf::per_output_plugin_instance_t,
     wf::signal::connection_t<wf::input_event_signal<wlr_pointer_button_event>> on_raw_pointer_button =
         [=] (wf::input_event_signal<wlr_pointer_button_event> *ev)
     {
-        if (ev->event->state == WLR_BUTTON_PRESSED)
+        if (ev->event->state == WL_POINTER_BUTTON_STATE_PRESSED)
         {
             last_input_press_position = get_global_input_coords();
         }
     };
 
-    wf::signal::connection_t<wf::input_event_signal<wlr_touch_down_event>> on_raw_touch_down =
-        [=] (wf::input_event_signal<wlr_touch_down_event> *ev)
+    wf::signal::connection_t<wf::post_input_event_signal<wlr_touch_down_event>> on_raw_touch_down =
+        [=] (wf::post_input_event_signal<wlr_touch_down_event> *ev)
     {
         if (ev->event->touch_id == 0)
         {
@@ -198,7 +198,7 @@ class wayfire_move : public wf::per_output_plugin_instance_t,
 
     void handle_pointer_button(const wlr_pointer_button_event& event) override
     {
-        if (event.state != WLR_BUTTON_RELEASED)
+        if (event.state != WL_POINTER_BUTTON_STATE_RELEASED)
         {
             return;
         }
@@ -372,6 +372,11 @@ class wayfire_move : public wf::per_output_plugin_instance_t,
      * is released at output-local coordinates (x, y) */
     wf::grid::slot_t calc_slot(wf::point_t point)
     {
+        if (!is_snap_enabled())
+        {
+            return wf::grid::SLOT_NONE;
+        }
+
         auto g = output->workarea->get_workarea();
         if (!(output->get_relative_geometry() & point))
         {
@@ -577,10 +582,7 @@ class wayfire_move : public wf::per_output_plugin_instance_t,
     void handle_input_motion()
     {
         drag_helper->handle_motion(get_global_input_coords());
-        if (is_snap_enabled())
-        {
-            update_slot(calc_slot(get_input_coords()));
-        }
+        update_slot(calc_slot(get_input_coords()));
     }
 
     void fini() override
