@@ -1,3 +1,4 @@
+#include <list>
 #include "wayfire/bindings.hpp"
 #include "wayfire/object.hpp"
 #include "wayfire/seat.hpp"
@@ -6,8 +7,8 @@
 #include "wayfire/signal-provider.hpp"
 #include "wayfire/util.hpp"
 #include "wayfire/view.hpp"
-#include "plugins/ipc/ipc-helpers.hpp"
-#include "plugins/ipc/ipc-method-repository.hpp"
+#include "wayfire/plugins/ipc/ipc-helpers.hpp"
+#include "wayfire/plugins/ipc/ipc-method-repository.hpp"
 #include "wayfire/plugins/common/cairo-util.hpp"
 #include "wayfire/plugins/common/shared-core-data.hpp"
 #include "wayfire/plugins/common/simple-text-node.hpp"
@@ -22,6 +23,7 @@
 #include <wayfire/config/types.hpp>
 #include <wayfire/output-layout.hpp>
 #include <wayfire/bindings-repository.hpp>
+#include <list>
 
 
 class wayfire_wsets_plugin_t : public wf::plugin_interface_t
@@ -66,33 +68,32 @@ class wayfire_wsets_plugin_t : public wf::plugin_interface_t
     std::list<wf::activator_callback> send_callback;
     std::map<int, std::shared_ptr<wf::workspace_set_t>> available_sets;
 
-    wf::ipc::method_callback set_output_wset = [=] (nlohmann::json data)
+    wf::ipc::method_callback set_output_wset = [=] (wf::json_t data)
     {
-        WFJSON_EXPECT_FIELD(data, "output-id", number_integer);
-        WFJSON_EXPECT_FIELD(data, "wset-index", number_integer);
-
-        wf::output_t *o = wf::ipc::find_output_by_id(data["output-id"]);
+        auto output_id  = wf::ipc::json_get_int64(data, "output-id");
+        auto wset_index = wf::ipc::json_get_int64(data, "wset-index");
+        wf::output_t *o = wf::ipc::find_output_by_id(output_id);
         if (!o)
         {
             return wf::ipc::json_error("output not found");
         }
 
-        select_workspace(data["wset-index"], o);
+        select_workspace(wset_index, o);
         return wf::ipc::json_ok();
     };
 
-    wf::ipc::method_callback send_view_to_wset = [=] (nlohmann::json data)
+    wf::ipc::method_callback send_view_to_wset = [=] (wf::json_t data)
     {
-        WFJSON_EXPECT_FIELD(data, "view-id", number_integer);
-        WFJSON_EXPECT_FIELD(data, "wset-index", number_integer);
+        auto view_id    = wf::ipc::json_get_int64(data, "view-id");
+        auto wset_index = wf::ipc::json_get_int64(data, "wset-index");
 
-        wayfire_toplevel_view view = toplevel_cast(wf::ipc::find_view_by_id(data["view-id"]));
+        wayfire_toplevel_view view = toplevel_cast(wf::ipc::find_view_by_id(view_id));
         if (!view)
         {
             return wf::ipc::json_error("view not found");
         }
 
-        send_window_to(data["wset-index"], view);
+        send_window_to(wset_index, view);
         return wf::ipc::json_ok();
     };
 
@@ -273,7 +274,11 @@ class wayfire_wsets_plugin_t : public wf::plugin_interface_t
         auto target_wset     = available_sets[index];
         const auto& old_wset = view->get_wset();
 
-        old_wset->remove_view(view);
+        if (old_wset)
+        {
+            old_wset->remove_view(view);
+        }
+
         wf::scene::remove_child(view->get_root_node());
         wf::emit_view_pre_moved_to_wset_pre(view, old_wset, target_wset);
 
