@@ -12,6 +12,7 @@
 #include "wayfire/bindings-repository.hpp"
 #include "wayfire/util.hpp"
 #include <memory>
+#include "wayfire/config-backend.hpp" // IWYU pragma: keep
 
 #include "plugin-loader.hpp"
 #include "seat/tablet.hpp"
@@ -75,6 +76,25 @@ struct wlr_idle_inhibitor_t : public wf::idle_inhibitor_t
     }
 };
 
+bool wf::compositor_core_t::is_gles2() const
+{
+    return wlr_renderer_is_gles2(renderer);
+}
+
+bool wf::compositor_core_t::is_vulkan() const
+{
+#if WLR_HAS_VULKAN_RENDERER
+    return wlr_renderer_is_vk(renderer);
+#else
+    return false;
+#endif
+}
+
+bool wf::compositor_core_t::is_pixman() const
+{
+    return wlr_renderer_is_pixman(renderer);
+}
+
 void wf::compositor_core_impl_t::init()
 {
     this->scene_root = std::make_shared<scene::root_node_t>();
@@ -94,9 +114,24 @@ void wf::compositor_core_impl_t::init()
     /* Needed for subsurfaces */
     wlr_subcompositor_create(display);
 
+    /* Legacy DRM */
+    if (runtime_config.legacy_wl_drm &&
+        wlr_renderer_get_texture_formats(renderer, WLR_BUFFER_CAP_DMABUF))
+    {
+        wlr_drm_create(display, renderer);
+    }
+
     protocols.data_device = wlr_data_device_manager_create(display);
-    protocols.primary_selection_v1 =
-        wlr_primary_selection_v1_device_manager_create(display);
+    wf::option_wrapper_t<bool> disable_primary_selection{"workarounds/disable_primary_selection"};
+    if (disable_primary_selection)
+    {
+        protocols.primary_selection_v1 = nullptr;
+    } else
+    {
+        protocols.primary_selection_v1 =
+            wlr_primary_selection_v1_device_manager_create(display);
+    }
+
     protocols.data_control = wlr_data_control_manager_v1_create(display);
 
     output_layout = std::make_unique<wf::output_layout_t>(backend);
@@ -127,23 +162,9 @@ void wf::compositor_core_impl_t::init()
         drm_lease_request.connect(&protocols.drm_v1->events.request);
     } else
     {
-        LOGE("Failed to create wlr_drm_lease_device_v1; VR will not be available!");
+        LOGI("Not using wlr_drm_lease_device_v1; VR will not be available!");
     }
 
-    /* input-inhibit setup */
-    protocols.input_inhibit = wlr_input_inhibit_manager_create(display);
-    input_inhibit_activated.set_callback([&] (void*)
-    {
-        input->set_exclusive_focus(protocols.input_inhibit->active_client);
-    });
-    input_inhibit_activated.connect(&protocols.input_inhibit->events.activate);
-
-    input_inhibit_deactivated.set_callback([&] (void*)
-    {
-        input->set_exclusive_focus(nullptr);
-    });
-    input_inhibit_deactivated.connect(&protocols.input_inhibit->events.deactivate);
-
     /* idle-inhibit setup */
     protocols.idle_notifier = wlr_idle_notifier_v1_create(display);
     protocols.idle_inhibit  = wlr_idle_inhibit_v1_create(display);
@@ -174,6 +195,11 @@ void wf::compositor_core_impl_t::init()
     {
         auto event = (wlr_virtual_pointer_v1_new_pointer_event*)data;
         auto ptr   = event->new_pointer;
+        if (event->suggested_output && !ptr->pointer.output_name)
+        {
+            ptr->pointer.output_name = strdup(event->suggested_output->name);
+        }
+
         input->handle_new_input(&ptr->pointer.base);
     });
     vptr_created.connect(&protocols.vptr_manager->events.new_virtual_pointer);
@@ -199,7 +225,9 @@ void wf::compositor_core_impl_t::init()
 
     im_relay = std::make_unique<input_method_relay>();
 
-    protocols.presentation = wlr_presentation_create(display, backend);
+    // TODO: is v2 the correct version here?
+    // https://gitlab.freedesktop.org/wlroots/wlroots/-/merge_requests/4858
+    protocols.presentation = wlr_presentation_create(display, backend, 2);
     protocols.viewporter   = wlr_viewporter_create(display);
 
     protocols.foreign_registry = wlr_xdg_foreign_registry_create(display);
@@ -213,20 +241,53 @@ void wf::compositor_core_impl_t::init()
 
     this->bindings = std::make_unique<bindings_repository_t>();
     image_io::init();
-    OpenGL::init();
+    if (is_gles2())
+    {
+        OpenGL::init();
+    }
+
+    increase_nofile_limit();
+
     this->state = compositor_state_t::START_BACKEND;
 }
 
+void wf::compositor_core_impl_t::increase_nofile_limit()
+{
+    if (getrlimit(RLIMIT_NOFILE, &user_maxfiles) != 0)
+    {
+        LOGE("Failed to getrlimit(RLIMIT_NOFILE), not increasing maximum open file descriptors. Might cause"
+             " crashes with many open views.");
+    } else
+    {
+        struct rlimit max_files = user_maxfiles;
+        max_files.rlim_cur = user_maxfiles.rlim_max;
+        if (setrlimit(RLIMIT_NOFILE, &max_files) != 0)
+        {
+            LOGE("Failed to setrlimit(RLIMIT_NOFILE), not increasing maximum open file descriptors. Might "
+                 "cause crashes with many open views.");
+        }
+    }
+}
+
+void wf::compositor_core_impl_t::restore_nofile_limit()
+{
+    if (setrlimit(RLIMIT_NOFILE, &user_maxfiles) != 0)
+    {
+        LOGE("Failed to setrlimit(RLIMIT_NOFILE), could not restore maximum open file descriptors.");
+    }
+}
+
 void wf::compositor_core_impl_t::post_init()
 {
     discard_command_output.load_option("workarounds/discard_command_output");
 
     core_backend_started_signal backend_started_ev;
     this->emit(&backend_started_ev);
-    this->state = compositor_state_t::RUNNING;
+    this->state = compositor_state_t::START_PLUGINS;
     plugin_mgr  = std::make_unique<wf::plugin_manager_t>();
     this->bindings->reparse_extensions();
 
+    this->state = compositor_state_t::RUNNING;
     // Move pointer to the middle of the leftmost, topmost output
     wf::pointf_t p;
     wf::output_t *wo = wf::get_core().output_layout->get_output_coords_at({FLT_MIN, FLT_MIN}, p);
@@ -235,7 +296,7 @@ void wf::compositor_core_impl_t::post_init()
     seat->focus_output(wo);
 
     // Refresh device mappings when we have all outputs and devices
-    input->refresh_device_mappings();
+    input->configure_input_devices();
 
     // Start processing cursor events
     seat->priv->cursor->setup_listeners();
@@ -245,6 +306,14 @@ void wf::compositor_core_impl_t::post_init()
 
 void wf::compositor_core_impl_t::shutdown()
 {
+    if (this->state < compositor_state_t::RUNNING)
+    {
+        // During initialization, shut down is a bit more complicated. We can deallocate core, but since we
+        // have not started the event loop, we can exit immediately.
+        deallocate_core();
+        std::exit(0);
+    }
+
     // We might get multiple signals in some scenarios. Shutdown only on the first instance.
     if (this->state != compositor_state_t::SHUTDOWN)
     {
@@ -273,12 +342,15 @@ void wf::compositor_core_impl_t::fini()
 
     LOGI("Unloading plugins...");
     plugin_mgr.reset();
+    _clear_data();
+
     // Shut down xwayland first, otherwise, wlroots will attempt to restart it when we kill it via
     // wl_display_destroy_clients().
     wf::fini_xwayland();
     LOGI("Stopping clients...");
     wl_display_destroy_clients(static_core->display);
     LOGI("Freeing resources...");
+    priv_output_layout_fini(output_layout.get());
     default_wm.reset();
     bindings.reset();
     scene_root.reset();
@@ -287,7 +359,6 @@ void wf::compositor_core_impl_t::fini()
     im_relay.reset();
     seat.reset();
     input.reset();
-    priv_output_layout_fini(output_layout.get());
     output_layout.reset();
     tx_manager.reset();
     OpenGL::fini();
@@ -388,9 +459,9 @@ wayfire_view wf::compositor_core_t::get_view_at(wf::pointf_t point)
     return isec ? node_to_view(isec->node->shared_from_this()) : nullptr;
 }
 
-wf::scene::node_ptr wf::compositor_core_impl_t::get_touch_focus()
+wf::scene::node_ptr wf::compositor_core_impl_t::get_touch_focus(int finger_id)
 {
-    return seat->priv->touch->get_focus();
+    return seat->priv->touch->get_focus(finger_id);
 }
 
 wayfire_view wf::compositor_core_t::get_touch_focus_view()
@@ -431,19 +502,29 @@ std::vector<wayfire_view> wf::compositor_core_t::get_all_views()
     return wf::tracking_allocator_t<view_interface_t>::get().get_all();
 }
 
+/**
+ * Upon successful execution, returns the PID of the child process.
+ * Returns 0 in case of failure.
+ */
 pid_t wf::compositor_core_impl_t::run(std::string command)
 {
     static constexpr size_t READ_END  = 0;
     static constexpr size_t WRITE_END = 1;
-    pid_t pid;
+
     int pipe_fd[2];
-    pipe2(pipe_fd, O_CLOEXEC);
+    int ret = pipe2(pipe_fd, O_CLOEXEC);
+    if (ret == -1)
+    {
+        LOGE("wf::compositor_core_impl_t::run: failed to create pipe2: ", strerror(errno));
+        return 0;
+    }
 
     /* The following is a "hack" for disowning the child processes,
      * otherwise they will simply stay as zombie processes */
-    pid = fork();
+    pid_t pid = fork();
     if (!pid)
     {
+        restore_nofile_limit();
         pid = fork();
         if (!pid)
         {
@@ -471,9 +552,9 @@ pid_t wf::compositor_core_impl_t::run(std::string command)
         } else
         {
             close(pipe_fd[READ_END]);
-            write(pipe_fd[WRITE_END], (void*)(&pid), sizeof(pid));
+            int ret = write(pipe_fd[WRITE_END], (void*)(&pid), sizeof(pid));
             close(pipe_fd[WRITE_END]);
-            _exit(0);
+            _exit(ret != sizeof(pid) ? 1 : 0);
         }
     } else
     {
@@ -482,8 +563,28 @@ pid_t wf::compositor_core_impl_t::run(std::string command)
         int status;
         waitpid(pid, &status, 0);
 
-        pid_t child_pid;
-        read(pipe_fd[READ_END], &child_pid, sizeof(child_pid));
+        // Return 0 if the child process didn't run or didn't exit normally, or returns a non-zero return
+        // value.
+        pid_t child_pid{};
+        if (WIFEXITED(status) && (WEXITSTATUS(status) == 0))
+        {
+            int ret = read(pipe_fd[READ_END], &child_pid, sizeof(child_pid));
+            if (ret != sizeof(child_pid))
+            {
+                // This is consider to be an error (even though theoretically a partial read would require an
+                // attempt to continue).
+                child_pid = 0;
+                if (ret == -1)
+                {
+                    LOGE("wf::compositor_core_impl_t::run(\"", command,
+                        "\"): failed to read PID from pipe end: ", strerror(errno));
+                } else
+                {
+                    LOGE("wf::compositor_core_impl_t::run(\"", command,
+                        "\"): short read of PID from pipe end, got ", std::to_string(ret), " bytes");
+                }
+            }
+        }
 
         close(pipe_fd[READ_END]);
 
@@ -511,6 +612,11 @@ void wf::start_move_view_to_wset(wayfire_toplevel_view v, std::shared_ptr<wf::wo
 
 void wf::move_view_to_output(wayfire_toplevel_view v, wf::output_t *new_output, bool reconfigure)
 {
+    if (v->get_output() == new_output)
+    {
+        return;
+    }
+
     wf::dassert(!v->parent, "Cannot move a dialog to a different output than its parent!");
 
     auto old_output = v->get_output();
@@ -582,7 +688,10 @@ const std::shared_ptr<wf::scene::root_node_t>& wf::compositor_core_impl_t::scene
 }
 
 wf::compositor_core_t::compositor_core_t()
-{}
+{
+    this->config = std::make_unique<wf::config::config_manager_t>();
+}
+
 wf::compositor_core_t::~compositor_core_t()
 {}
 
@@ -631,3 +740,8 @@ std::unique_ptr<wf::compositor_core_impl_t> wf::compositor_core_impl_t::static_c
 
 // TODO: move this to a better location
 wf_runtime_config runtime_config;
+
+std::shared_ptr<wf::config::option_base_t> wf::detail::load_raw_option(const std::string& name)
+{
+    return wf::get_core().config->get_option(name);
+}
