@@ -11,9 +11,12 @@ void init();
 /** Destroy the default GL program and resources */
 void fini();
 /** Indicate we have started repainting the given output */
-void bind_output(wf::output_t *output, uint32_t fb);
+void bind_output(uint32_t fb);
 /** Indicate the output frame has been finished */
-void unbind_output(wf::output_t *output);
+void unbind_output();
+
+/** Debugging: if GL_CALL experiences an error, exit immediately and print stacktrace. */
+extern bool exit_on_gles_error;
 }
 
 #endif /* end of include guard: WF_OPENGL_PRIV_HPP */
