@@ -6,6 +6,7 @@
 #include "wayfire/render-manager.hpp"
 #include "wayfire/signal-definitions.hpp"
 #include "wayfire/util.hpp"
+#include "wayfire/config-backend.hpp"
 
 #include "../output/output-impl.hpp"
 #include <xf86drmMode.h>
@@ -21,39 +22,107 @@
 
 static void*const WF_NOOP_OUTPUT_MAGIC = (void*)0x1234;
 
+// wlroots wrappers
+namespace wf
+{
+class wlr_output_state_setter_t
+{
+  public:
+    wlr_output_state_setter_t()
+    {
+        wlr_output_state_init(&pending);
+    }
+
+    ~wlr_output_state_setter_t()
+    {
+        wlr_output_state_finish(&pending);
+    }
+
+    wlr_output_state_setter_t(const wlr_output_state_setter_t&) = delete;
+    wlr_output_state_setter_t(wlr_output_state_setter_t&&) = delete;
+    wlr_output_state_setter_t& operator =(const wlr_output_state_setter_t&) = delete;
+    wlr_output_state_setter_t& operator =(wlr_output_state_setter_t&&) = delete;
+
+    void reset()
+    {
+        wlr_output_state_finish(&pending);
+        wlr_output_state_init(&pending);
+    }
+
+    // Commit all pending changes on the output.
+    // Returns true if the output was successfully committed.
+    //
+    // After this operation, the pending output state (@pending) is reset.
+    bool commit(wlr_output *output)
+    {
+        bool r = wlr_output_commit_state(output, &pending);
+        reset();
+        return r;
+    }
+
+    // Test whether the pending state can be applied on the output.
+    bool test(wlr_output *output)
+    {
+        return wlr_output_test_state(output, &pending);
+    }
+
+    // Test whether the pending state can be applied on the output.
+    // If so, commit the state.
+    //
+    // In both cases, reset @pending.
+    bool test_and_commit(wlr_output *output)
+    {
+        if (test(output))
+        {
+            commit(output);
+            return true;
+        } else
+        {
+            reset();
+            return false;
+        }
+    }
+
+    wlr_output_state pending;
+};
+}
+
+static std::map<std::string, wl_output_transform> output_transforms = {
+    {"normal", WL_OUTPUT_TRANSFORM_NORMAL},
+    {"90", WL_OUTPUT_TRANSFORM_90},
+    {"180", WL_OUTPUT_TRANSFORM_180},
+    {"270", WL_OUTPUT_TRANSFORM_270},
+    {"flipped", WL_OUTPUT_TRANSFORM_FLIPPED},
+    {"90_flipped", WL_OUTPUT_TRANSFORM_FLIPPED_90},
+    {"180_flipped", WL_OUTPUT_TRANSFORM_FLIPPED_180},
+    {"270_flipped", WL_OUTPUT_TRANSFORM_FLIPPED_270},
+};
+
 static wl_output_transform get_transform_from_string(std::string transform)
 {
-    if (transform == "normal")
+    auto it = output_transforms.find(transform);
+    if (it != output_transforms.end())
     {
-        return WL_OUTPUT_TRANSFORM_NORMAL;
-    } else if (transform == "90")
-    {
-        return WL_OUTPUT_TRANSFORM_90;
-    } else if (transform == "180")
-    {
-        return WL_OUTPUT_TRANSFORM_180;
-    } else if (transform == "270")
-    {
-        return WL_OUTPUT_TRANSFORM_270;
-    } else if (transform == "flipped")
-    {
-        return WL_OUTPUT_TRANSFORM_FLIPPED;
-    } else if (transform == "180_flipped")
-    {
-        return WL_OUTPUT_TRANSFORM_FLIPPED_180;
-    } else if (transform == "90_flipped")
-    {
-        return WL_OUTPUT_TRANSFORM_FLIPPED_90;
-    } else if (transform == "270_flipped")
-    {
-        return WL_OUTPUT_TRANSFORM_FLIPPED_270;
+        return it->second;
     }
 
     LOGE("Bad output transform in config: ", transform);
-
     return WL_OUTPUT_TRANSFORM_NORMAL;
 }
 
+static std::string wl_transform_to_string(wl_output_transform transform)
+{
+    for (auto& it : output_transforms)
+    {
+        if (it.second == transform)
+        {
+            return it.first;
+        }
+    }
+
+    return "normal";
+}
+
 wlr_output_mode *find_matching_mode(wlr_output *output,
     const wlr_output_mode& reference, bool exact_match = false)
 {
@@ -258,6 +327,7 @@ static const char *get_format_name(uint32_t format)
 struct output_layout_output_t
 {
     wlr_output *handle;
+    wlr_output_state_setter_t pending_state;
     output_state_t current_state{};
     bool is_externally_managed = false;
     bool is_nested_compositor  = false;
@@ -266,7 +336,7 @@ struct output_layout_output_t
     int current_bit_depth = RENDER_BIT_DEPTH_DEFAULT;
 
     std::unique_ptr<wf::output_impl_t> output;
-    wl_listener_wrapper on_destroy, on_commit;
+    wl_listener_wrapper on_destroy, on_commit, on_request_state;
 
     std::shared_ptr<wf::config::section_t> config_section;
     wf::option_wrapper_t<wf::output_config::mode_t> mode_opt;
@@ -295,6 +365,7 @@ struct output_layout_output_t
     {
         this->handle = handle;
         on_destroy.connect(&handle->events.destroy);
+        on_request_state.connect(&handle->events.request_state);
         initialize_config_options();
 
         is_nested_compositor = wlr_output_is_wl(handle);
@@ -457,9 +528,6 @@ struct output_layout_output_t
         wf::output_config::mode_t mode = mode_opt;
         wlr_output_mode tmp;
 
-        LOGI("loaded mode ",
-            ((wf::option_sptr_t<wf::output_config::mode_t>)mode_opt)->get_value_str());
-
         switch (mode.get_type())
         {
           case output_config::MODE_AUTO:
@@ -618,8 +686,7 @@ struct output_layout_output_t
                 (current_bit_depth == current_state.depth))
             {
                 /* Commit the enabling of the output */
-                wlr_output_commit(handle);
-
+                pending_state.commit(handle);
                 return;
             }
         }
@@ -628,7 +695,7 @@ struct output_layout_output_t
         auto built_in = find_matching_mode(handle, mode, custom_mode);
         if (built_in)
         {
-            wlr_output_set_mode(handle, built_in);
+            wlr_output_state_set_mode(&pending_state.pending, built_in);
         } else
         {
             LOGI("Couldn't find matching mode ",
@@ -636,24 +703,22 @@ struct output_layout_output_t
                 " for output ", handle->name, ". Trying to use custom mode",
                 "(might not work)");
 
-            wlr_output_set_custom_mode(handle, mode.width, mode.height, mode.refresh);
+            wlr_output_state_set_custom_mode(&pending_state.pending, mode.width, mode.height, mode.refresh);
         }
 
-        wlr_output_commit(handle);
+        pending_state.commit(handle);
 
         const bool adaptive_sync_enabled = (handle->adaptive_sync_status == WLR_OUTPUT_ADAPTIVE_SYNC_ENABLED);
 
         if (adaptive_sync_enabled != current_state.vrr)
         {
-            wlr_output_enable_adaptive_sync(handle, current_state.vrr);
-            if (wlr_output_test(handle))
+            wlr_output_state_set_adaptive_sync_enabled(&pending_state.pending, current_state.vrr);
+            if (pending_state.test_and_commit(handle))
             {
-                wlr_output_commit(handle);
                 LOGD("Changed adaptive sync on output: ", handle->name, " to ", current_state.vrr);
             } else
             {
                 LOGE("Failed to change adaptive sync on output: ", handle->name);
-                wlr_output_rollback(handle);
             }
         }
 
@@ -661,10 +726,9 @@ struct output_layout_output_t
         {
             for (auto fmt : formats_for_depth[current_state.depth])
             {
-                wlr_output_set_render_format(handle, fmt);
-                if (wlr_output_test(handle))
+                wlr_output_state_set_render_format(&pending_state.pending, fmt);
+                if (pending_state.test_and_commit(handle))
                 {
-                    wlr_output_commit(handle);
                     current_bit_depth = current_state.depth;
                     LOGD("Set output format to ", get_format_name(fmt), " on output ", handle->name);
                     break;
@@ -683,15 +747,27 @@ struct output_layout_output_t
     /** Render the output using texture as source */
     void render_output(wlr_texture *texture)
     {
-        auto renderer = get_core().renderer;
-        wlr_output_attach_render(handle, NULL);
-        wlr_renderer_begin(renderer, handle->width, handle->height);
+        // TODO: use render-manager's functions, apply gamma, use our normal pass functions.
+        struct wlr_render_pass *pass = wlr_output_begin_render_pass(handle, &pending_state.pending, NULL);
+        if (pass == NULL)
+        {
+            return;
+        }
 
-        wf::texture_t tex{texture};
-        OpenGL::render_transformed_texture(tex, {-1, -1, 2, 2});
+        // Render other output as a fullscreen texture.
+        wlr_render_texture_options opts{};
+        opts.texture = texture;
+        opts.alpha   = NULL;
+        opts.blend_mode  = WLR_RENDER_BLEND_MODE_NONE;
+        opts.filter_mode = WLR_SCALE_FILTER_BILINEAR;
+        opts.clip    = NULL;
+        opts.src_box = {0, 0, 0, 0};
+        opts.dst_box = {0, 0, handle->width, handle->height};
+        opts.transform = WL_OUTPUT_TRANSFORM_NORMAL;
+        wlr_render_pass_add_texture(pass, &opts);
 
-        wlr_renderer_end(renderer);
-        wlr_output_commit(handle);
+        wlr_render_pass_submit(pass);
+        pending_state.commit(handle);
     }
 
     /* Load output contents and render them */
@@ -728,10 +804,10 @@ struct output_layout_output_t
 
     void set_enabled(bool enabled)
     {
-        wlr_output_enable(handle, enabled);
+        wlr_output_state_set_enabled(&pending_state.pending, enabled);
         if (!enabled)
         {
-            wlr_output_commit(handle);
+            pending_state.commit(handle);
         }
     }
 
@@ -903,15 +979,15 @@ struct output_layout_output_t
         {
             if (handle->transform != state.transform)
             {
-                wlr_output_set_transform(handle, state.transform);
+                wlr_output_state_set_transform(&pending_state.pending, state.transform);
             }
 
             if (handle->scale != state.scale)
             {
-                wlr_output_set_scale(handle, state.scale);
+                wlr_output_state_set_scale(&pending_state.pending, state.scale);
             }
 
-            wlr_output_commit(handle);
+            pending_state.commit(handle);
 
             ensure_wayfire_output(get_effective_size());
             output->render->damage_whole();
@@ -995,10 +1071,10 @@ class output_layout_t::impl
         on_backend_destroy.set_callback([=] (auto) { deinit_noop(); });
         on_backend_destroy.connect(&wf::get_core().renderer->events.destroy);
 
-        output_layout = wlr_output_layout_create();
+        output_layout = wlr_output_layout_create(get_core().display);
         get_core().connect(&on_config_reload);
 
-        noop_backend = wlr_headless_backend_create(get_core().display);
+        noop_backend = wlr_headless_backend_create(get_core().ev_loop);
         wlr_backend_start(noop_backend);
 
         get_core().connect(&on_backend_started);
@@ -1086,8 +1162,8 @@ class output_layout_t::impl
             state.scale     = head->state.scale;
             state.transform = head->state.transform;
             state.vrr = head->state.adaptive_sync_enabled;
-            if ((handle->pending.render_format == DRM_FORMAT_XRGB2101010) ||
-                (handle->pending.render_format == DRM_FORMAT_XBGR2101010))
+            if ((handle->render_format == DRM_FORMAT_XRGB2101010) ||
+                (handle->render_format == DRM_FORMAT_XBGR2101010))
             {
                 state.depth = 10;
             } else
@@ -1122,7 +1198,10 @@ class output_layout_t::impl
 
     void ensure_noop_output()
     {
-        LOGI("new output: NOOP-1");
+        if (!noop_output || (noop_output->current_state.source != OUTPUT_IMAGE_SOURCE_SELF))
+        {
+            LOGI("new output: NOOP-1");
+        }
 
         if (!noop_output)
         {
@@ -1145,7 +1224,14 @@ class output_layout_t::impl
          * next reconfiguration. This is needed because if we are removing
          * an output, we might get into a situation where the last physical
          * output has already been removed but we are yet to add the noop one */
-        noop_output->apply_state(noop_output->load_configured_state());
+        auto configured_state = noop_output->load_configured_state();
+        if (configured_state.source != OUTPUT_IMAGE_SOURCE_SELF)
+        {
+            LOGW("Noop output cannot be disabled, mirrored or put into DPMS state!");
+            configured_state.source = OUTPUT_IMAGE_SOURCE_SELF;
+        }
+
+        noop_output->apply_state(configured_state);
         wlr_output_layout_add_auto(output_layout, noop_output->handle);
         timer_remove_noop.disconnect();
     }
@@ -1168,8 +1254,7 @@ class output_layout_t::impl
         state.source = OUTPUT_IMAGE_SOURCE_NONE;
         noop_output->apply_state(state);
         wlr_output_layout_remove(output_layout, noop_output->handle);
-        // Trigger repositioning of all outputs
-        apply_configuration(get_current_configuration());
+        emit_configuration_changed_for_dynamic_outputs(get_current_configuration());
     }
 
     void add_output(wlr_output *output)
@@ -1189,6 +1274,8 @@ class output_layout_t::impl
             return;
         }
 
+        LOGI("Adding with ", get_core().renderer);
+
         if (!wlr_output_init_render(output,
             get_core().allocator, get_core().renderer))
         {
@@ -1203,6 +1290,29 @@ class output_layout_t::impl
             remove_output(output);
         });
 
+        lo->on_request_state.set_callback([=] (void *data)
+        {
+            auto ev = static_cast<wlr_output_event_request_state*>(data);
+            wlr_output_commit_state(ev->output, ev->state);
+            if (!lo->output)
+            {
+                return;
+            }
+
+            send_wlr_configuration();
+            for (auto& output : outputs)
+            {
+                output.second->emit_configuration_changed(
+                    wf::OUTPUT_POSITION_CHANGE | wf::OUTPUT_MODE_CHANGE);
+            }
+
+            output_layout_configuration_changed_signal signal;
+            wf::get_core().output_layout->emit(&signal);
+
+            lo->output->render->damage_whole();
+            lo->output->render->schedule_redraw();
+        });
+
         reconfigure_from_config();
     }
 
@@ -1423,9 +1533,67 @@ class output_layout_t::impl
         return ok;
     }
 
+    static std::string_view get_output_source_name(output_image_source_t source)
+    {
+        switch (source)
+        {
+          case OUTPUT_IMAGE_SOURCE_INVALID:
+            return "invalid";
+
+          case OUTPUT_IMAGE_SOURCE_SELF:
+            return "self";
+
+          case OUTPUT_IMAGE_SOURCE_NONE:
+            return "none";
+
+          case OUTPUT_IMAGE_SOURCE_DPMS:
+            return "dpms";
+
+          case OUTPUT_IMAGE_SOURCE_MIRROR:
+            return "mirror";
+        }
+
+        return "unknown";
+    }
+
     /** Apply the given configuration. Config MUST be a valid configuration */
     void apply_configuration(const output_configuration_t& config)
     {
+        LOGC(OUTPUT, "Applying configuration:");
+        for (auto& entry : config)
+        {
+            LOGC(OUTPUT, "\toutput: ", entry.first->name);
+            if (entry.second.position.is_automatic_position())
+            {
+                LOGC(OUTPUT, "\t  position: automatic");
+            } else
+            {
+                LOGC(OUTPUT, "\t  position: ",
+                    entry.second.position.get_x(), ", ", entry.second.position.get_y());
+            }
+
+            if (entry.second.source == OUTPUT_IMAGE_SOURCE_NONE)
+            {
+                LOGC(OUTPUT, "\t  mode: off");
+            } else if (entry.second.source == OUTPUT_IMAGE_SOURCE_DPMS)
+            {
+                LOGC(OUTPUT, "\t  mode: dpms");
+            } else if (entry.second.source == OUTPUT_IMAGE_SOURCE_MIRROR)
+            {
+                LOGC(OUTPUT, "\t  mode: mirror ", entry.second.mirror_from);
+            } else
+            {
+                LOGC(OUTPUT, "\t  mode: ",
+                    entry.second.mode.width, "x", entry.second.mode.height, "@", entry.second.mode.refresh,
+                    entry.second.uses_custom_mode ? " (custom)" : "");
+            }
+
+            LOGC(OUTPUT, "\t  scale: ", entry.second.scale);
+            LOGC(OUTPUT, "\t  transform: ", wl_transform_to_string(entry.second.transform));
+            LOGC(OUTPUT, "\t  vrr: ", entry.second.vrr);
+            LOGC(OUTPUT, "\t  depth: ", entry.second.depth);
+        }
+
         /* The order in which we enable and disable outputs is important.
          * Firstly, on some systems where there aren't enough CRTCs, we can
          * only enable a subset of all outputs at once. This means we should
@@ -1468,7 +1636,7 @@ class output_layout_t::impl
             auto& state  = entry.second;
             auto& lo     = this->outputs[handle];
 
-            if (!(state.source & OUTPUT_IMAGE_SOURCE_SELF))
+            if (state.source == OUTPUT_IMAGE_SOURCE_NONE)
             {
                 /* First shut down the output, move its views, etc. while it
                  * is still in the output layout and its global is active.
@@ -1534,21 +1702,7 @@ class output_layout_t::impl
 
         /* Fifth: emit configuration-changed again for dynamically-positioned outputs, because their position
          * might have changed. */
-        for (auto& entry : config)
-        {
-            auto& handle = entry.first;
-            auto& state  = entry.second;
-            auto& lo     = this->outputs[handle];
-
-            if (state.source & OUTPUT_IMAGE_SOURCE_SELF &&
-                entry.second.position.is_automatic_position())
-            {
-                lo->emit_configuration_changed(wf::OUTPUT_POSITION_CHANGE);
-            }
-        }
-
-        wf::output_layout_configuration_changed_signal ev;
-        get_core().output_layout->emit(&ev);
+        emit_configuration_changed_for_dynamic_outputs(config);
 
         if (count_enabled > 0)
         {
@@ -1569,6 +1723,25 @@ class output_layout_t::impl
         });
     }
 
+    void emit_configuration_changed_for_dynamic_outputs(const output_configuration_t& config)
+    {
+        for (auto& entry : config)
+        {
+            auto& handle = entry.first;
+            auto& state  = entry.second;
+            auto& lo     = this->outputs[handle];
+
+            if (state.source & OUTPUT_IMAGE_SOURCE_SELF &&
+                entry.second.position.is_automatic_position())
+            {
+                lo->emit_configuration_changed(wf::OUTPUT_POSITION_CHANGE);
+            }
+        }
+
+        wf::output_layout_configuration_changed_signal ev;
+        get_core().output_layout->emit(&ev);
+    }
+
     void send_wlr_configuration()
     {
         auto wlr_configuration = wlr_output_configuration_v1_create();
