@@ -1,12 +1,61 @@
-#include "bindings-repository.hpp"
 #include <wayfire/core.hpp>
 #include <algorithm>
+#include "bindings-repository-impl.hpp"
+
+wf::bindings_repository_t::bindings_repository_t()
+{
+    priv = std::make_unique<impl>();
+    wf::get_core().connect(&priv->on_config_reload);
+}
+
+template<class Option, class Callback>
+static void push_binding(wf::binding_container_t<Option, Callback>& bindings,
+    wf::option_sptr_t<Option> opt, Callback *callback)
+{
+    auto bnd = std::make_unique<wf::binding_t<Option, Callback>>();
+    bnd->activated_by = opt;
+    bnd->callback     = callback;
+    bindings.emplace_back(std::move(bnd));
+}
+
+wf::bindings_repository_t::~bindings_repository_t()
+{}
+
+void wf::bindings_repository_t::add_key(option_sptr_t<keybinding_t> key, wf::key_callback *cb)
+{
+    push_binding(priv->keys, key, cb);
+}
+
+void wf::bindings_repository_t::add_axis(option_sptr_t<keybinding_t> axis, wf::axis_callback *cb)
+{
+    push_binding(priv->axes, axis, cb);
+}
+
+void wf::bindings_repository_t::add_button(option_sptr_t<buttonbinding_t> button, wf::button_callback *cb)
+{
+    push_binding(priv->buttons, button, cb);
+}
+
+void wf::bindings_repository_t::add_activator(
+    option_sptr_t<activatorbinding_t> activator, wf::activator_callback *cb)
+{
+    push_binding(priv->activators, activator, cb);
+    if (activator->get_value().get_hotspots().size())
+    {
+        priv->recreate_hotspots();
+    }
+}
 
 bool wf::bindings_repository_t::handle_key(const wf::keybinding_t& pressed,
     uint32_t mod_binding_key)
 {
+    if (!priv->enabled)
+    {
+        return false;
+    }
+
     std::vector<std::function<bool()>> callbacks;
-    for (auto& binding : this->keys)
+    for (auto& binding : this->priv->keys)
     {
         if (binding->activated_by->get_value() == pressed)
         {
@@ -20,7 +69,7 @@ bool wf::bindings_repository_t::handle_key(const wf::keybinding_t& pressed,
         }
     }
 
-    for (auto& binding : this->activators)
+    for (auto& binding : this->priv->activators)
     {
         if (binding->activated_by->get_value().has_match(pressed))
         {
@@ -57,9 +106,14 @@ bool wf::bindings_repository_t::handle_key(const wf::keybinding_t& pressed,
 bool wf::bindings_repository_t::handle_axis(uint32_t modifiers,
     wlr_pointer_axis_event *ev)
 {
+    if (!priv->enabled)
+    {
+        return false;
+    }
+
     std::vector<wf::axis_callback*> callbacks;
 
-    for (auto& binding : this->axes)
+    for (auto& binding : this->priv->axes)
     {
         if (binding->activated_by->get_value() == wf::keybinding_t{modifiers, 0})
         {
@@ -77,8 +131,13 @@ bool wf::bindings_repository_t::handle_axis(uint32_t modifiers,
 
 bool wf::bindings_repository_t::handle_button(const wf::buttonbinding_t& pressed)
 {
+    if (!priv->enabled)
+    {
+        return false;
+    }
+
     std::vector<std::function<bool()>> callbacks;
-    for (auto& binding : this->buttons)
+    for (auto& binding : this->priv->buttons)
     {
         if (binding->activated_by->get_value() == pressed)
         {
@@ -92,7 +151,7 @@ bool wf::bindings_repository_t::handle_button(const wf::buttonbinding_t& pressed
         }
     }
 
-    for (auto& binding : this->activators)
+    for (auto& binding : this->priv->activators)
     {
         if (binding->activated_by->get_value().has_match(pressed))
         {
@@ -121,8 +180,13 @@ bool wf::bindings_repository_t::handle_button(const wf::buttonbinding_t& pressed
 
 void wf::bindings_repository_t::handle_gesture(const wf::touchgesture_t& gesture)
 {
+    if (!priv->enabled)
+    {
+        return;
+    }
+
     std::vector<std::function<void()>> callbacks;
-    for (auto& binding : this->activators)
+    for (auto& binding : this->priv->activators)
     {
         if (binding->activated_by->get_value().has_match(gesture))
         {
@@ -146,19 +210,34 @@ void wf::bindings_repository_t::handle_gesture(const wf::touchgesture_t& gesture
     }
 }
 
-bool wf::bindings_repository_t::handle_activator(
-    const std::string& activator, const wf::activator_data_t& data)
+bool wf::bindings_repository_t::handle_extension_generic(
+    std::function<bool(const std::any& stored_tag)> callback, const wf::activator_data_t& data)
 {
-    auto opt = wf::get_core().config.get_option(activator);
-    for (auto& act : this->activators)
+    if (!priv->enabled)
     {
-        if (act->activated_by == opt)
+        return false;
+    }
+
+    std::vector<std::function<bool()>> callbacks;
+    for (auto& binding : this->priv->activators)
+    {
+        for (auto& tag : binding->tags)
         {
-            return (*act->callback)(data);
+            if (callback(tag))
+            {
+                callbacks.emplace_back([cb = binding->callback, &data] () { return (*cb)(data); });
+                break;
+            }
         }
     }
 
-    return false;
+    bool handled = false;
+    for (auto& cb : callbacks)
+    {
+        handled |= cb();
+    }
+
+    return handled;
 }
 
 void wf::bindings_repository_t::rem_binding(void *callback)
@@ -173,49 +252,61 @@ void wf::bindings_repository_t::rem_binding(void *callback)
         container.erase(it, container.end());
     };
 
-    erase(keys);
-    erase(buttons);
-    erase(axes);
-    erase(activators);
+    bool update_hotspots = false;
+    for (auto& act : this->priv->activators)
+    {
+        update_hotspots |= !act->activated_by->get_value().get_hotspots().empty();
+    }
 
-    recreate_hotspots();
+    erase(priv->keys);
+    erase(priv->buttons);
+    erase(priv->axes);
+    erase(priv->activators);
+
+    if (update_hotspots)
+    {
+        priv->recreate_hotspots();
+    }
 }
 
-void wf::bindings_repository_t::rem_binding(binding_t *binding)
+void wf::bindings_repository_t::set_enabled(bool enabled)
 {
-    const auto& erase = [binding] (auto& container)
+    priv->enabled += (enabled ? 1 : -1);
+    priv->recreate_hotspots();
+}
+
+void wf::bindings_repository_t::impl::reparse_extensions()
+{
+    for (auto& binding : this->activators)
     {
-        auto it = std::remove_if(container.begin(), container.end(),
-            [binding] (const auto& ptr)
+        binding->tags.clear();
+    }
+
+    idle_reparse_bindings.run_once([=]
+    {
+        for (auto& binding : this->activators)
         {
-            return ptr.get() == binding;
-        });
-        container.erase(it, container.end());
-    };
+            auto value = binding->activated_by->get_value();
+            for (auto& ext : value.get_extensions())
+            {
+                parse_activator_extension_signal data;
+                data.extension_binding = ext;
+                wf::get_core().emit(&data);
 
-    erase(keys);
-    erase(buttons);
-    erase(axes);
-    erase(activators);
-
-    recreate_hotspots();
-}
-
-wf::bindings_repository_t::bindings_repository_t(wf::output_t *output) :
-    hotspot_mgr(output)
-{
-    on_config_reload.set_callback([=] (wf::signal_data_t*)
-    {
-        recreate_hotspots();
-    });
-
-    wf::get_core().connect_signal("reload-config", &on_config_reload);
-}
-
-void wf::bindings_repository_t::recreate_hotspots()
-{
-    this->idle_recreate_hotspots.run_once([=] ()
-    {
-        hotspot_mgr.update_hotspots(activators);
+                if (data.tag.has_value())
+                {
+                    binding->tags.push_back(data.tag);
+                } else
+                {
+                    LOGW("Failed to parse extension binding ", ext, " from option ",
+                        binding->activated_by->get_name());
+                }
+            }
+        }
     });
 }
+
+void wf::bindings_repository_t::reparse_extensions()
+{
+    priv->reparse_extensions();
+}
