@@ -120,6 +120,7 @@ void wf::cursor_t::init_xcursor()
 
     if (xcursor)
     {
+        last_cursor_name.clear(); // make sure we set the new cursor image with the new xcursor_manager
         wlr_xcursor_manager_destroy(xcursor);
     }
 
@@ -144,6 +145,13 @@ void wf::cursor_t::set_cursor(std::string name)
         name = "left_ptr";
     }
 
+    if (name == last_cursor_name)
+    {
+        return;
+    }
+
+    last_cursor_name = name;
+
     idle_set_cursor.run_once([name, this] ()
     {
         wlr_cursor_set_xcursor(cursor, xcursor, name.c_str());
@@ -165,6 +173,7 @@ void wf::cursor_t::hide_cursor()
     idle_set_cursor.disconnect();
     wlr_cursor_set_surface(cursor, NULL, 0, 0);
     this->hide_ref_counter++;
+    last_cursor_name.clear();
 }
 
 void wf::cursor_t::warp_cursor(wf::pointf_t point)
@@ -200,6 +209,8 @@ void wf::cursor_t::set_cursor(
     }
 
     wlr_cursor_set_surface(cursor, ev->surface, ev->hotspot_x, ev->hotspot_y);
+
+    last_cursor_name.clear();
 }
 
 void wf::cursor_t::set_touchscreen_mode(bool enabled)
