@@ -1,14 +1,11 @@
 #ifndef INPUT_MANAGER_HPP
 #define INPUT_MANAGER_HPP
 
-#include <map>
+#include <wayfire/output-layout.hpp>
 #include <vector>
-#include <chrono>
 
 #include "seat-impl.hpp"
-#include "wayfire/plugin.hpp"
 #include "wayfire/signal-provider.hpp"
-#include "wayfire/view.hpp"
 #include "wayfire/core.hpp"
 #include "wayfire/signal-definitions.hpp"
 #include <wayfire/option-wrapper.hpp>
@@ -37,11 +34,17 @@ class input_manager_t
      */
     uint32_t locked_mods = 0;
 
+    /**
+     * Map a single input device to output as specified in the
+     * config file or by hints in the wlroots backend.
+     */
+    void configure_input_device(std::unique_ptr<wf::input_device_impl_t> & device);
+
     /**
      * Go through all input devices and map them to outputs as specified in the
      * config file or by hints in the wlroots backend.
      */
-    void refresh_device_mappings();
+    void configure_input_devices();
 
     input_manager_t();
     ~input_manager_t() = default;
