@@ -1,7 +1,9 @@
 #pragma once
 
 #include <chrono>
-#include "seat.hpp"
+#include "seat-impl.hpp"
+#include "wayfire/signal-definitions.hpp"
+#include "wayfire/signal-provider.hpp"
 #include "wayfire/util.hpp"
 #include <wayfire/option-wrapper.hpp>
 
@@ -31,15 +33,17 @@ class keyboard_t
     /* The keycode which triggered the modifier binding */
     uint32_t mod_binding_key = 0;
 
+    /** Convert a key to a modifier */
+    uint32_t mod_from_key(uint32_t key);
+
   private:
     wf::wl_listener_wrapper on_key, on_modifier;
     void setup_listeners();
 
-    wf::signal_connection_t on_config_reload;
+    wf::signal::connection_t<wf::reload_config_signal> on_config_reload;
     void reload_input_options();
 
-    wf::option_wrapper_t<std::string>
-    model, variant, layout, options, rules;
+    wf::option_wrapper_t<std::string> model, variant, layout, options, rules;
     wf::option_wrapper_t<int> repeat_rate, repeat_delay;
     /** Options have changed in the config file */
     bool dirty_options = true;
@@ -47,10 +51,6 @@ class keyboard_t
     std::chrono::steady_clock::time_point mod_binding_start;
 
     bool handle_keyboard_key(uint32_t key, uint32_t state);
-    void handle_keyboard_mod(uint32_t key, uint32_t state);
-
-    /** Convert a key to a modifier */
-    uint32_t mod_from_key(uint32_t key);
 
     /** Get the current locked mods */
     uint32_t get_locked_mods();
