@@ -2,11 +2,14 @@
 #define WF_SEAT_TABLET_HPP
 
 #include <wayfire/util.hpp>
-#include "seat.hpp"
+#include "seat-impl.hpp"
+#include "wayfire/object.hpp"
 #include "wayfire/signal-definitions.hpp"
+#include "wayfire/signal-provider.hpp"
 
 namespace wf
 {
+struct tablet_t;
 struct tablet_tool_t
 {
     /**
@@ -18,14 +21,17 @@ struct tablet_tool_t
 
     wlr_tablet_tool *tool;
     wlr_tablet_v2_tablet_tool *tool_v2;
+    tablet_t *tablet;
 
     /**
      * Called whenever a refocus of the tool is necessary
      */
-    void update_tool_position();
+    void update_tool_position(bool real_update);
 
     /** Set the proximity surface */
-    void set_focus(wf::surface_interface_t *surface);
+    bool set_focus(scene::node_ptr node);
+
+    void reset_grab();
 
     /**
      * Send the axis updates directly.
@@ -47,16 +53,17 @@ struct tablet_tool_t
   private:
     wf::wl_listener_wrapper on_destroy, on_set_cursor;
     wf::wl_listener_wrapper on_tool_v2_destroy;
-    wf::signal_connection_t on_surface_map_state_changed;
-    wf::signal_connection_t on_views_updated;
+
+    wf::signal::connection_t<wf::scene::root_node_update_signal>
+    on_root_node_updated;
 
     /** Tablet that this tool belongs to */
     wlr_tablet_v2_tablet *tablet_v2;
 
     /** Surface where the tool is in */
-    wf::surface_interface_t *proximity_surface = nullptr;
+    scene::node_ptr proximity_surface = nullptr;
     /** Surface where the tool was grabbed */
-    wf::surface_interface_t *grabbed_surface = nullptr;
+    scene::node_ptr grabbed_node = nullptr;
 
     double tilt_x = 0.0;
     double tilt_y = 0.0;
@@ -74,6 +81,7 @@ struct tablet_t : public input_device_impl_t
     tablet_t(wlr_cursor *cursor, wlr_input_device *tool);
     virtual ~tablet_t();
 
+    void reconfigure_device(std::shared_ptr<wf::config::section_t> device_section) override;
     /** Handle a tool tip event */
     void handle_tip(wlr_tablet_tool_tip_event *ev,
         input_event_processing_mode_t mode);
@@ -88,6 +96,7 @@ struct tablet_t : public input_device_impl_t
         input_event_processing_mode_t mode);
 
     wlr_tablet_v2_tablet *tablet_v2;
+    std::vector<std::unique_ptr<tablet_tool_t>> tools_list;
 
   private:
     wlr_tablet *handle;
@@ -98,13 +107,15 @@ struct tablet_t : public input_device_impl_t
      * The wayfire tool will be created if it doesn't exist yet.
      */
     tablet_tool_t *ensure_tool(wlr_tablet_tool *tool);
+
+    bool should_use_absolute_positioning(wlr_tablet_tool *tool);
 };
 
 struct tablet_pad_t : public input_device_impl_t
 {
   public:
     tablet_pad_t(wlr_input_device *pad);
-    ~tablet_pad_t();
+    ~tablet_pad_t() = default;
 
   private:
     wlr_tablet_v2_tablet_pad *pad_v2;
@@ -113,7 +124,11 @@ struct tablet_pad_t : public input_device_impl_t
     nonstd::observer_ptr<tablet_t> attached_to;
 
     wf::wl_listener_wrapper on_attach, on_button, on_strip, on_ring;
-    signal_callback_t on_input_devices_changed, on_keyboard_focus_changed;
+    wf::wl_listener_wrapper on_focus_destroy;
+
+    wf::signal::connection_t<wf::input_device_added_signal> on_device_added;
+    wf::signal::connection_t<wf::input_device_removed_signal> on_device_removed;
+    wf::signal::connection_t<wf::keyboard_focus_changed_signal> on_keyboard_focus_changed;
 
     /** Attach the pad to the given tablet. */
     void attach_to_tablet(tablet_t *tablet);
