@@ -102,11 +102,14 @@ bool wayfire_xdg_popup::should_close_on_focus_change(wf::keyboard_focus_changed_
 
 wayfire_xdg_popup::wayfire_xdg_popup(wlr_xdg_popup *popup) : wf::view_interface_t()
 {
-    this->popup_parent = wf::wl_surface_to_wayfire_view(popup->parent->resource).get();
+    auto parent_ptr = wf::wl_surface_to_wayfire_view(popup->parent->resource);
+    wf::dassert(parent_ptr.get(), "Popup has no existing parent?");
+    this->popup_parent = parent_ptr->weak_from_this();
+
     this->popup = popup;
     this->role  = wf::VIEW_ROLE_UNMANAGED;
 
-    if (!dynamic_cast<wayfire_xdg_popup*>(popup_parent.get()))
+    if (!dynamic_cast<wayfire_xdg_popup*>(parent_ptr.get()))
     {
         // 'toplevel' popups are responsible for closing their popup tree when the parent loses focus.
         // Note: we shouldn't close nested popups manually, since the parent popups will destroy them as well.
@@ -120,9 +123,11 @@ wayfire_xdg_popup::wayfire_xdg_popup(wlr_xdg_popup *popup) : wf::view_interface_
     }
 
     on_surface_commit.set_callback([&] (void*) { commit(); });
+    // We'll receive the initial commit soon
+    on_surface_commit.connect(&popup->base->surface->events.commit);
 
-    LOGI("New xdg popup");
-    this->main_surface = std::make_shared<wf::scene::wlr_surface_node_t>(popup->base->surface, true);
+    LOGC(VIEWS, "New xdg popup");
+    this->main_surface = std::make_shared<wf::scene::wlr_surface_node_t>(popup->base->surface, false);
 
     on_map.set_callback([&] (void*) { map(); });
     on_unmap.set_callback([&] (void*) { unmap(); });
@@ -152,16 +157,16 @@ wayfire_xdg_popup::wayfire_xdg_popup(wlr_xdg_popup *popup) : wf::view_interface_
     });
     parent_app_id_changed.set_callback([=] (auto)
     {
-        this->handle_app_id_changed(popup_parent->get_app_id());
+        this->handle_app_id_changed(popup_parent.lock()->get_app_id());
     });
     parent_title_changed.set_callback([=] (auto)
     {
-        this->handle_title_changed(popup_parent->get_title());
+        this->handle_title_changed(popup_parent.lock()->get_title());
     });
 
-    popup_parent->connect(&this->parent_geometry_changed);
-    popup_parent->connect(&this->parent_app_id_changed);
-    popup_parent->connect(&this->parent_title_changed);
+    parent_ptr->connect(&this->parent_geometry_changed);
+    parent_ptr->connect(&this->parent_app_id_changed);
+    parent_ptr->connect(&this->parent_title_changed);
 }
 
 wayfire_xdg_popup::~wayfire_xdg_popup() = default;
@@ -172,8 +177,7 @@ std::shared_ptr<wayfire_xdg_popup> wayfire_xdg_popup::create(wlr_xdg_popup *popu
 
     self->surface_root_node = std::make_shared<wayfire_xdg_popup_node>(self);
     self->set_surface_root_node(self->surface_root_node);
-    self->set_output(self->popup_parent->get_output());
-    self->unconstrain();
+    self->set_output(self->popup_parent.lock()->get_output());
     return self;
 }
 
@@ -186,18 +190,34 @@ void wayfire_xdg_popup::map()
         return;
     }
 
+    // Update main surface before everything else.
+    // Note that map() happens before commit(), so the next commit will just set the same surface state
+    // again.
+    main_surface->apply_current_surface_state();
     update_position();
 
-    wf::scene::layer parent_layer = wf::get_view_layer(popup_parent).value_or(wf::scene::layer::WORKSPACE);
+    auto parent = popup_parent.lock();
+    wf::scene::layer parent_layer = wf::get_view_layer(parent)
+        .value_or(wf::scene::layer::WORKSPACE);
     auto target_layer = wf::scene::layer::UNMANAGED;
     if ((int)parent_layer > (int)wf::scene::layer::WORKSPACE)
     {
         target_layer = parent_layer;
     }
 
-    wf::scene::readd_front(get_output()->node_for_layer(target_layer), get_root_node());
+    wf::scene::floating_inner_ptr target_parent_node = get_output()->node_for_layer(target_layer);
+    if (std::dynamic_pointer_cast<wayfire_xdg_popup>(parent))
+    {
+        target_parent_node = parent->get_root_node();
+    }
+
+    wf::scene::readd_front(target_parent_node, get_root_node());
+
+    if (!popup->base->initial_commit)
+    {
+        on_surface_commit.connect(&popup->base->surface->events.commit);
+    }
 
-    on_surface_commit.connect(&popup->base->surface->events.commit);
     priv->set_mapped_surface_contents(main_surface);
     priv->set_mapped(main_surface->get_surface());
     priv->set_enabled(true);
@@ -237,13 +257,21 @@ void wayfire_xdg_popup::unmap()
 
 void wayfire_xdg_popup::commit()
 {
+    // Update main surface.
+    main_surface->apply_current_surface_state();
+    if (popup->base->initial_commit)
+    {
+        unconstrain();
+    }
+
     update_size();
     update_position();
 }
 
 void wayfire_xdg_popup::update_position()
 {
-    if (!popup_parent->is_mapped() || !popup)
+    auto parent = popup_parent.lock();
+    if (!parent || !popup)
     {
         return;
     }
@@ -256,10 +284,8 @@ void wayfire_xdg_popup::update_position()
 
     if (wlr_xdg_surface *xdg_surface = wlr_xdg_surface_try_from_wlr_surface(popup->parent))
     {
-        wlr_box box;
-        wlr_xdg_surface_get_geometry(xdg_surface, &box);
-        local_offset.x += box.x;
-        local_offset.y += box.y;
+        local_offset.x += xdg_surface->geometry.x;
+        local_offset.y += xdg_surface->geometry.y;
     }
 
     wf::pointf_t popup_offset = wf::place_popup_at(popup->parent, popup->base->surface, local_offset);
@@ -274,7 +300,7 @@ void wayfire_xdg_popup::unconstrain()
         auto as_popup = dynamic_cast<wayfire_xdg_popup*>(toplevel_parent);
         if (as_popup)
         {
-            toplevel_parent = as_popup->popup_parent.get();
+            toplevel_parent = as_popup->popup_parent.lock().get();
         } else
         {
             break;
@@ -300,6 +326,7 @@ void wayfire_xdg_popup::destroy()
     on_new_popup.disconnect();
     on_ping_timeout.disconnect();
     on_reposition.disconnect();
+    popup->base->data = nullptr;
     popup = nullptr;
 }
 
@@ -389,6 +416,15 @@ wf::geometry_t wayfire_xdg_popup::get_geometry()
 
 wlr_surface*wayfire_xdg_popup::get_keyboard_focus_surface()
 {
+    static wf::option_wrapper_t<bool> focus_main_view{"workarounds/focus_main_surface_instead_of_popup"};
+    if (focus_main_view)
+    {
+        if (auto parent = this->popup_parent.lock())
+        {
+            return parent->get_keyboard_focus_surface();
+        }
+    }
+
     return priv->wsurface;
 }
 
@@ -408,7 +444,7 @@ class xdg_popup_controller_t
             view->destroy();
             delete this;
         });
-        on_destroy.connect(&popup->base->events.destroy);
+        on_destroy.connect(&popup->events.destroy);
         view = wayfire_xdg_popup::create(popup);
     }
 
@@ -433,21 +469,22 @@ static wf::wl_listener_wrapper on_xdg_created;
 
 void wf::init_xdg_shell()
 {
-    xdg_handle = wlr_xdg_shell_create(wf::get_core().display, 3);
+    xdg_handle = wlr_xdg_shell_create(wf::get_core().display, XDG_TOPLEVEL_STATE_SUSPENDED_SINCE_VERSION);
     if (xdg_handle)
     {
         on_xdg_created.set_callback([&] (void *data)
         {
-            auto surf = static_cast<wlr_xdg_surface*>(data);
+            auto surf = static_cast<wlr_xdg_toplevel*>(data);
             wf::new_xdg_surface_signal new_xdg_surf;
-            new_xdg_surf.surface = surf;
-            wf::get_core().emit(&new_xdg_surf);
-            if (new_xdg_surf.use_default_implementation && (surf->role == WLR_XDG_SURFACE_ROLE_TOPLEVEL))
+            new_xdg_surf.surface = surf->base;
+            wf::get_core().emit(
+                &new_xdg_surf);
+            if (new_xdg_surf.use_default_implementation && (surf->base->role == WLR_XDG_SURFACE_ROLE_TOPLEVEL))
             {
-                default_handle_new_xdg_toplevel(surf->toplevel);
+                default_handle_new_xdg_toplevel(surf);
             }
         });
-        on_xdg_created.connect(&xdg_handle->events.new_surface);
+        on_xdg_created.connect(&xdg_handle->events.new_toplevel);
     }
 }
 
