@@ -49,7 +49,7 @@ wf::xdg_toplevel_view_base_t::xdg_toplevel_view_base_t(wlr_xdg_toplevel *topleve
         wf::view_implementation::emit_ping_timeout_signal(self());
     });
 
-    on_destroy.connect(&xdg_toplevel->base->events.destroy);
+    on_destroy.connect(&xdg_toplevel->events.destroy);
     on_new_popup.connect(&xdg_toplevel->base->events.new_popup);
     on_ping_timeout.connect(&xdg_toplevel->base->events.ping_timeout);
     on_set_title.connect(&xdg_toplevel->events.set_title);
@@ -243,6 +243,11 @@ wf::xdg_toplevel_view_t::xdg_toplevel_view_t(wlr_xdg_toplevel *tlvl) : xdg_tople
     on_request_minimize.connect(&xdg_toplevel->events.request_minimize);
     on_show_window_menu.connect(&xdg_toplevel->events.request_show_window_menu);
     on_request_fullscreen.connect(&xdg_toplevel->events.request_fullscreen);
+
+    if (xdg_toplevel && uses_csd.count(xdg_toplevel->base->surface))
+    {
+        this->has_client_decoration = uses_csd[xdg_toplevel->base->surface];
+    }
 }
 
 std::shared_ptr<wf::xdg_toplevel_view_t> wf::xdg_toplevel_view_t::create(wlr_xdg_toplevel *toplevel)
@@ -302,11 +307,6 @@ bool wf::xdg_toplevel_view_t::is_mapped() const
 
 void wf::xdg_toplevel_view_t::map()
 {
-    if (xdg_toplevel && uses_csd.count(xdg_toplevel->base->surface))
-    {
-        this->has_client_decoration = uses_csd[xdg_toplevel->base->surface];
-    }
-
     adjust_view_output_on_map(this);
 
     xdg_toplevel_view_base_t::map();
@@ -424,20 +424,30 @@ struct wf_xdg_decoration_t
             mode = default_mode;
         }
 
-        wlr_xdg_toplevel_decoration_v1_set_mode(decor, mode);
+        if (decor->toplevel->base->initialized)
+        {
+            wlr_xdg_toplevel_decoration_v1_set_mode(decor, mode);
+        }
     };
 
     std::function<void(void*)> commit = [&] (void*)
     {
+        wlr_xdg_surface *xdg_surface = decor->toplevel->base;
+
         bool use_csd = (decor->current.mode == WLR_XDG_TOPLEVEL_DECORATION_V1_MODE_CLIENT_SIDE);
-        uses_csd[decor->toplevel->base->surface] = use_csd;
+        uses_csd[xdg_surface->surface] = use_csd;
 
         auto wf_surface = dynamic_cast<wf::xdg_toplevel_view_t*>(
-            wf::wl_surface_to_wayfire_view(decor->toplevel->base->surface->resource).get());
+            wf::wl_surface_to_wayfire_view(xdg_surface->surface->resource).get());
         if (wf_surface)
         {
             wf_surface->set_decoration_mode(use_csd);
         }
+
+        if (decor->toplevel->base->initial_commit)
+        {
+            mode_request(NULL);
+        }
     };
 
     wf_xdg_decoration_t(wlr_xdg_toplevel_decoration_v1 *_decor) :
@@ -454,8 +464,6 @@ struct wf_xdg_decoration_t
         on_mode_request.connect(&decor->events.request_mode);
         on_commit.connect(&decor->toplevel->base->surface->events.commit);
         on_destroy.connect(&decor->events.destroy);
-        /* Read initial decoration settings */
-        mode_request(NULL);
     }
 };
 
@@ -517,9 +525,7 @@ void wf::fini_xdg_decoration_handlers()
 void wf::xdg_toplevel_view_t::start_map_tx()
 {
     LOGC(VIEWS, "Start mapping ", self());
-    wlr_box box;
-    wlr_xdg_surface_get_geometry(xdg_toplevel->base, &box);
-
+    wlr_box box  = xdg_toplevel->base->geometry;
     auto margins = wtoplevel->pending().margins;
     box.x = wtoplevel->pending().geometry.x + margins.left;
     box.y = wtoplevel->pending().geometry.y + margins.top;
@@ -556,7 +562,7 @@ class xdg_toplevel_controller_t
     xdg_toplevel_controller_t(wlr_xdg_toplevel *toplevel)
     {
         on_destroy.set_callback([=] (auto) { delete this; });
-        on_destroy.connect(&toplevel->base->events.destroy);
+        on_destroy.connect(&toplevel->events.destroy);
         view = wf::xdg_toplevel_view_t::create(toplevel);
 
         on_map.set_callback([=] (void*)
