@@ -26,6 +26,7 @@ class wayfire_xwayland_view : public wf::toplevel_view_interface_t, public wayfi
     wf::wl_listener_wrapper on_request_move, on_request_resize,
         on_request_maximize, on_request_minimize, on_request_activate,
         on_request_fullscreen, on_set_hints, on_set_parent;
+    wf::wl_listener_wrapper on_surface_commit;
 
     wf::wl_listener_wrapper on_set_decorations;
     std::shared_ptr<wf::xw::xwayland_toplevel_t> toplevel;
@@ -226,7 +227,7 @@ class wayfire_xwayland_view : public wf::toplevel_view_interface_t, public wayfi
         {
             wf::view_hints_changed_signal data;
             data.view = this;
-            if (xw->hints->flags & XCB_ICCCM_WM_HINT_X_URGENCY)
+            if (xw->hints && (xw->hints->flags & XCB_ICCCM_WM_HINT_X_URGENCY))
             {
                 data.demands_attention = true;
             }
@@ -318,9 +319,9 @@ class wayfire_xwayland_view : public wf::toplevel_view_interface_t, public wayfi
             };
 
             /* Make sure geometry is properly visible on the view output */
+            save_geometry = save_geometry - wf::origin(get_output()->get_layout_geometry());
             save_geometry = wf::clamp(save_geometry, get_output()->workarea->get_workarea());
-            toplevel->pending().geometry = save_geometry;
-            wf::get_core().default_wm->update_last_windowed_geometry({this});
+            wf::get_core().default_wm->update_last_windowed_geometry({this}, save_geometry);
         }
     }
 
@@ -365,6 +366,7 @@ class wayfire_xwayland_view : public wf::toplevel_view_interface_t, public wayfi
         toplevel->set_main_surface(nullptr);
         toplevel->pending().mapped = false;
         wf::get_core().tx_manager->schedule_object(toplevel);
+        on_surface_commit.disconnect();
     }
 
     void map(wlr_surface *surface)
@@ -374,8 +376,72 @@ class wayfire_xwayland_view : public wf::toplevel_view_interface_t, public wayfi
         wf::adjust_view_output_on_map(this);
         do_map(surface, false);
         on_surface_commit.connect(&surface->events.commit);
+        bool wants_focus = false;
+
+        switch (wlr_xwayland_surface_icccm_input_model(xw))
+        {
+          /*
+           * Abbreviated from ICCCM section 4.1.7 (Input Focus):
+           *
+           * Passive Input - The client expects keyboard input but never
+           * explicitly sets the input focus.
+           * Locally Active Input - The client expects keyboard input and
+           * explicitly sets the input focus, but it only does so when one
+           * of its windows already has the focus.
+           *
+           * Passive and Locally Active clients set the input field of
+           * WM_HINTS to True, which indicates that they require window
+           * manager assistance in acquiring the input focus.
+           */
+          case WLR_ICCCM_INPUT_MODEL_PASSIVE:
+          case WLR_ICCCM_INPUT_MODEL_LOCAL:
+            wants_focus = true;
+            break;
+
+          /*
+           * Globally Active Input - The client expects keyboard input and
+           * explicitly sets the input focus, even when it is in windows
+           * the client does not own. ... It wants to prevent the window
+           * manager from setting the input focus to any of its windows
+           * [because it may or may not want focus].
+           *
+           * Globally Active client windows may receive a WM_TAKE_FOCUS
+           * message from the window manager. If they want the focus, they
+           * should respond with a SetInputFocus request.
+           */
+          case WLR_ICCCM_INPUT_MODEL_GLOBAL:
+            /*
+             * Assume that NORMAL and DIALOG windows are likely to
+             * want focus. These window types should show up in the
+             * Alt-Tab switcher and be automatically focused when
+             * they become topmost.
+             *
+             * When we offer focus, we don't really focus the dialog.
+             * Instead, by offering focus like that, we let the client
+             * switch focus between its surfaces without involving the
+             * compositor.
+             */
+            if (wlr_xwayland_surface_has_window_type(xw,
+                WLR_XWAYLAND_NET_WM_WINDOW_TYPE_NORMAL) ||
+                wlr_xwayland_surface_has_window_type(xw,
+                    WLR_XWAYLAND_NET_WM_WINDOW_TYPE_DIALOG))
+            {
+                wlr_xwayland_surface_offer_focus(xw);
+            }
+
+            break;
+
+          /*
+           * No Input - The client never expects keyboard input.
+           *
+           * No Input and Globally Active clients set the input field to
+           * False, which requests that the window manager not set the
+           * input focus to their top-level window.
+           */
+          case WLR_ICCCM_INPUT_MODEL_NONE:
+            break;
+        }
 
-        const bool wants_focus = (wlr_xwayland_icccm_input_model(xw) != WLR_ICCCM_INPUT_MODEL_NONE);
         if (wants_focus)
         {
             wf::get_core().default_wm->focus_request(self());
@@ -389,7 +455,6 @@ class wayfire_xwayland_view : public wf::toplevel_view_interface_t, public wayfi
     {
         LOGC(VIEWS, "Do unmap ", self());
         do_unmap();
-        on_surface_commit.disconnect();
     }
 
     void commit()
